import logging
import json
from typing import Dict, List, Any
from flask import current_app, request
from elasticsearch8 import Elasticsearch


def main() -> str:
    """Process and index weather observation data into Elasticsearch.

    Handles:
    - Elasticsearch client initialization with security credentials
    - Suppression of SSL warnings for self-signed certificates
    - Bulk indexing of observation records
    - Document ID generation using station ID and timestamp
    - Request payload validation and logging

    Returns:
        'ok' on successful processing of all observations

    Raises:
        JSONDecodeError: If invalid JSON payload received
        ElasticsearchException: For indexing failures
    """
    # Initialize Elasticsearch client
    es_client: Elasticsearch = Elasticsearch(
        'https://elasticsearch-master.elastic.svc.cluster.local:9200',
        verify_certs=False,
        ssl_show_warn=False,
        basic_auth=('elastic', 'elastic')
    )

    # Validate and parse request payload
    request_data: List[Dict[str, Any]] = request.get_json(force=True)
    current_app.logger.info(f'Processing {len(request_data)} observations')

    # Index each observation
    for observation in request_data:
        doc_id: str = f'{observation["stationid"]}-{observation["timestamp"]}'
        
        index_response: Dict[str, Any] = es_client.index(
            index='observations',
            id=doc_id,
            body=observation
        )
        
        current_app.logger.info(
            f'Indexed observation {doc_id} - '
            f'Version: {index_response["_version"]}'
        )

    return 'ok'
