import logging
import json
from typing import Optional, Dict, Any
from flask import current_app, request
from elasticsearch8 import Elasticsearch
from string import Template

# Elasticsearch query templates with typed parameters
days_expr: Template = Template('''{
    "range": {
        "timestamp": {
            "gte": "${date}T00:00:00",
            "lte": "${date}T23:59:59"
        }
    }
}''')

station_expr: Template = Template('''{
    "bool": {
        "must": [
            {"match": {"stationid": "${station}"}},
            {"range": {
                "timestamp": {
                    "gte": "${date}T00:00:00",
                    "lte": "${date}T23:59:59"
                }
            }}
        ]
    }
}''')

def main() -> Dict[str, Any]:
    """Calculate daily temperature averages from weather observations.

    Handles:
    - Date filtering with optional station filtering
    - Elasticsearch aggregation queries
    - Header parameter extraction

    Returns:
        JSON response containing:
        - average_temperature: Calculated daily average
        - observation_count: Number of measurements
        - time_range: Query timeframe

    Raises:
        KeyError: If required date parameter is missing
        ElasticsearchException: For query failures
    """
    # Extract and validate headers
    req: Request = request
    date: Optional[str] = req.headers.get('X-Fission-Params-Date')
    station: Optional[str] = req.headers.get('X-Fission-Params-Station')

    if not date:
        current_app.logger.error('Missing required date parameter')
        return {'error': 'Date parameter required'}, 400

    # Initialize Elasticsearch client with type annotation
    es_client: Elasticsearch = Elasticsearch(
        'https://elasticsearch-master.elastic.svc.cluster.local:9200',
        verify_certs=False,
        ssl_show_warn=False,
        basic_auth=('elastic', 'elastic')
    )

    # Build dynamic query based on parameters
    query_body: Dict[str, Any] = {
        "query": {
            "bool": {
                "filter": [
                    json.loads(station_expr.substitute(date=date, station=station))
                    if station else
                    json.loads(days_expr.substitute(date=date))
                ]
            }
        },
        "aggs": {
            "avg_temp": {"avg": {"field": "air_temp"}},
            "total_obs": {"value_count": {"field": "air_temp"}}
        }
    }

    current_app.logger.info(
        f'Executing temperature aggregation for '
        f'Date: {date}' +
        (f' Station: {station}' if station else '')
    )

    # Execute search with type annotated result
    res: Dict[str, Any] = es_client.search(
        index='observations*',
        body=query_body
    )

    # Extract and format results
    agg_data: Dict[str, Any] = res.get('aggregations', {})
    return {
        'average_temperature': agg_data.get('avg_temp', {}).get('value'),
        'observation_count': agg_data.get('total_obs', {}).get('value'),
        'time_range': f'{date}T00:00:00 to {date}T23:59:59'
    }
