import unittest
import requests
import json
from unittest.mock import patch

class HTTPSession:
    """HTTP client session wrapper for making API requests.

    Attributes:
        session: Requests session object
        base_url: Base URL for API endpoints
    """
    
    def __init__(self, protocol: str, hostname: str, port: int) -> None:
        self.session = requests.Session()
        self.base_url = f'{protocol}://{hostname}:{port}'

    def get(self, path: str) -> requests.Response:
        """Send GET request to specified path."""
        return self.session.get(f'{self.base_url}/{path}')

    def post(self, path: str, data: dict) -> requests.Response:
        """Send POST request with data to specified path."""
        return self.session.post(f'{self.base_url}/{path}', data)

    def put(self, path: str, data: dict) -> requests.Response:
        """Send PUT request with data to specified path."""
        return self.session.put(f'{self.base_url}/{path}', data)

    def delete(self, path: str) -> requests.Response:
        """Send DELETE request to specified path."""
        return self.session.delete(f'{self.base_url}/{path}')

class TestEnd2End(unittest.TestCase):
    """End-to-end tests for student/course management API."""
    
    def test_student(self) -> None:
        """Test student resource CRUD operations."""
        self.assertEqual(test_request.get('/students/111').status_code, 200)
        self.assertEqual(test_request.get('/students/111').text, 'student')

        self.assertEqual(test_request.put('/students/111', {}).status_code, 200)
        self.assertEqual(test_request.put('/students/111', {}).text, 'student')

        self.assertEqual(test_request.delete('/students/111').status_code, 200)
        self.assertEqual(test_request.delete('/students/111').text, 'student')

    def test_students(self) -> None:
        """Test students collection endpoint."""
        self.assertEqual(test_request.get('/students').status_code, 200)
        self.assertEqual(test_request.get('/students').text, 'students')

    def test_course(self) -> None:
        """Test course resource CRUD operations."""
        self.assertEqual(test_request.get('/courses/111').status_code, 200)
        self.assertEqual(test_request.get('/courses/111').text, 'course')

        self.assertEqual(test_request.put('/courses/111', {}).status_code, 200)
        self.assertEqual(test_request.put('/courses/111', {}).text, 'course')

        self.assertEqual(test_request.delete('/courses/111').status_code, 200)
        self.assertEqual(test_request.delete('/courses/111').text, 'course')

    def test_courses(self) -> None:
        """Test courses collection and related endpoints."""
        self.assertEqual(test_request.get('/courses').status_code, 200)
        self.assertEqual(test_request.get('/courses').text, 'courses')

        self.assertEqual(test_request.get('/courses/111/students').status_code, 200)
        self.assertEqual(test_request.get('/courses/111/students').text, 'coursesstudents')

if __name__ == '__main__':

    test_request = HTTPSession('http', 'localhost', 9090)
    unittest.main()
