from flask import Request, current_app
import requests, logging, json
from typing import Dict, Any, Tuple, Optional

def config(k: str) -> str:
    """Read configuration parameter from default configs.
    
    Args:
        k: Configuration key filename to read
        
    Returns:
        String contents of the configuration file
    """
    with open(f'/configs/default/parameters/{k}', 'r') as f:
        return f.read()


def main() -> Tuple[Optional[Dict[str, Any]], int]:
    """Search for course documents in Elasticsearch with specific fields.
    
    Queries ES for:
    - Documents where type='course'
    - Returns only id, timestamp, and name fields
    - Sorted by name ascending
    
    Returns:
        Tuple containing:
        - Parsed JSON response (None on error)
        - HTTP status code
    """
    try:
        es_url = f'{config("ES_URL")}/{config("ES_DATABASE")}/_search'
        
        # Elasticsearch query to filter and sort courses
        query_body = {
            '_source': False,
            'query': {
                'term': {
                    'type': {'value': 'course'}
                }
            },
            'fields': [
                {'field': 'id'},
                {'field': 'timestamp'},
                {'field': 'name'}
            ],
            'sort': [{
                'name': {
                    'order': 'asc',
                    'missing': '_last',
                    'unmapped_type': 'keyword'
                }
            }]
        }

        r: requests.Response = requests.post(
            es_url,
            verify=False,
            auth=(config('ES_USERNAME'), config('ES_PASSWORD')),
            headers={'Content-type': 'application/json'},
            data=json.dumps(query_body)
        )
        return r.json(), r.status_code

    except Exception as e:
        current_app.logger.error(str(e))
        return {'message': f'Error: {str(e)}'}, 500
