from flask import Request, current_app
from typing import Tuple, Dict, Any, Optional
import requests
import logging
import json
from Commons import Commons
from ESCourse import ESCourse

ResponseTuple = Tuple[Dict[str, Any], int]

def main() -> ResponseTuple:
    """Handle course document operations in Elasticsearch.

    Supports:
    - GET: Retrieve course document
    - PUT: Create/update course document with optimistic concurrency
    - DELETE: Remove course document

    Returns:
        Tuple containing:
        - JSON response body
        - HTTP status code

    Raises:
        Exception: Logs error and returns 500 on failure
    """
    try:
        course: ESCourse = ESCourse(Commons(), request)

        if request.method == 'GET':
            response: requests.Response = requests.get(
                course.url(),
                verify=False,
                auth=Commons().auth()
            )
            return response.json(), response.status_code

        elif request.method == 'PUT':
            # Get current document version
            get_response = requests.get(
                course.url(),
                verify=False,
                auth=Commons().auth()
            )
            params: Dict[str, int] = {}
            if get_response.status_code == 200:
                params = {
                    'if_seq_no': get_response.json()['_seq_no'],
                    'if_primary_term': get_response.json()['_primary_term']
                }

            # Update document
            put_response = requests.put(
                course.url(),
                verify=False,
                auth=Commons().auth(),
                headers={'Content-type': 'application/json'},
                data=json.dumps(course.add_fields(request.get_json(silent=True))),
                params=params
            )
            return put_response.json(), put_response.status_code

        elif request.method == 'DELETE':
            # Verify document exists
            get_response = requests.get(
                course.url(),
                verify=False,
                auth=Commons().auth()
            )
            if get_response.status_code != 200:
                return get_response.json(), get_response.status_code

            # Delete document
            del_response = requests.delete(
                course.url(),
                verify=False,
                auth=Commons().auth(),
                params={
                    'if_seq_no': get_response.json()['_seq_no'],
                    'if_primary_term': get_response.json()['_primary_term']
                }
            )
            return del_response.json(), del_response.status_code

    except Exception as e:
        current_app.logger.error(str(e))
        return {'message': f'Error: {str(e)}'}, 500

    return {'message': 'Method not allowed'}, 405
