from flask import Request, current_app
from typing import Tuple, Dict, Any, Optional
import requests
import logging
import json
from Commons import Commons
from ESDocument import ESDocument

ResponseTuple = Tuple[Dict[str, Any], int]

def main(request: Request) -> ResponseTuple:
    """Handle course-student relationship operations in Elasticsearch.

    Supports:
    - GET: Retrieve students enrolled in a specific course
    - POST: Not supported
    - DELETE: Not supported

    Args:
        request: Flask request object containing headers with 'X-Fission-Params-Courseid'

    Returns:
        Tuple containing Elasticsearch response and status code

    Raises:
        Exception: Returns 500 on data processing failures
    """
    try:
        commons = Commons()
        
        # Build Elasticsearch query
        query_body = {
            '_source': False,
            'query': {
                'bool': {
                    'must': [
                        {'term': {'courses': request.headers['X-Fission-Params-Courseid']}},
                        {'term': {'type': 'student'}}
                    ]
                }
            },
            'fields': [{'field': 'id'}, {'field': 'timestamp'}, {'field': 'name'}],
            'sort': [{
                'name': {
                    'order': 'asc',
                    'missing': '_last',
                    'unmapped_type': 'keyword'
                }
            }]
        }

        # Execute search request
        response: requests.Response = requests.post(
            commons.search_url(),
            verify=False,
            auth=commons.auth(),
            headers={'Content-type': 'application/json'},
            data=json.dumps(query_body)
        )
        return response.json(), response.status_code

    except Exception as e:
        current_app.logger.error(str(e))
        return {'message': f'Query failed: {str(e)}'}, 500
