from flask import Request
from typing import Any, Dict
from Commons import Commons
import requests, logging, json
from datetime import datetime

class ESDocument:
    """Base class for Elasticsearch document operations.

    Provides common functionality for:
    - Constructing document URLs
    - Validating required fields
    - Formatting document data
    """

    def __init__(self, commons: Commons, req: Request, doc_type: str) -> None:
        """Initialize base document handler.

        Args:
            commons: Application configuration utilities
            req: Flask request object containing document data
            doc_type: Elasticsearch document type (e.g. 'course', 'student')
        """
        self.commons = commons
        self.req = req
        self.doc_type = doc_type

    def url(self) -> str:
        """Build document-specific Elasticsearch endpoint URL.

        Returns:
            Formatted URL string combining base URL and document ID
        """
        return f"{self.commons.config('ES_URL')}/{self.commons.config('ES_DATABASE')}/_doc/{self.doc_type}-{self.req.headers.get('X-Fission-Params-Id')}"

    def add_fields(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """Enhance document data with METADATA fields.

        Args:
            data: Raw document data from request

        Returns:
            Modified data with additional metadata fields
        """
        data['timestamp'] = datetime.now().isoformat()
        data['id'] = self.req.headers.get(f'X-Fission-Params-{self.doc_type.capitalize()}id')
        data['type'] = self.doc_type
        return data

    def get(self) -> tuple:
        """Retrieve document from Elasticsearch.

        Returns:
            Tuple containing response data and HTTP status code
        """
        r = requests.get(self.url(), verify=False, auth=self.commons.auth())
        return r.json(), r.status_code

    def put(self) -> tuple:
        """Update document in Elasticsearch.

        Returns:
            Tuple containing response data and HTTP status code
        """
        r = requests.get(self.url(), verify=False, auth=self.commons.auth())

        if r.status_code == 200:
            params={'if_seq_no': r.json()['_seq_no'], 'if_primary_term': r.json()['_primary_term']}
        else:
            params= {}

        r = requests.put(self.url(), verify=False, auth=self.commons.auth(),
                headers={'Content-type': 'application/json'},
                data=json.dumps(self.add_fields(request.json)),
                params=params)
        return r.json(), r.status_code

    def delete(self) -> tuple:
        """Delete document from Elasticsearch.

        Returns:
            Tuple containing response data and HTTP status code
        """
        r = requests.get(self.url(), verify=False, auth=self.commons.auth())

        if r.status_code != 200:
            return r.json(), r.status_code

        r = requests.delete(self.url(), verify=False, auth=self.commons.auth(),
                params={'if_seq_no': r.json()['_seq_no'], 'if_primary_term': r.json()['_primary_term']})
        return r.json(), r.status_code

    def all(self) -> str:
        """Retrieve all documents of a specific type from Elasticsearch.

        Returns:
            JSON-formatted query string
        """
        return json.dumps({'_source': False,
                    'query': {
                        'term': {
                            'type': {
                                'value': f'{self.doc_type}'
                            }
                        }
                    },
                    'fields':[
                        {'field':'id'},
                        {'field':'timestamp'},
                        {'field':'name'},
                        {'field':'courses'}
                    ],
                    'sort': [
                        {
                          'name': {
                            'order': 'asc',
                            'missing': '_last',
                            'unmapped_type': 'keyword'
                         }
                       }
                    ]
                    })
