import unittest
import requests
import json
import time
from typing import Dict, Any

class HTTPSession:
    """HTTP client for making REST API requests during testing.

    Handles session management and provides basic CRUD operations:
    - GET, POST, PUT, DELETE
    """
    
    def __init__(self, protocol: str, hostname: str, port: int) -> None:
        """Initialize test session with base URL.

        Args:
            protocol: http/https
            hostname: Server domain/IP
            port: Service port number
        """
        self.session = requests.Session()
        self.base_url = f'{protocol}://{hostname}:{port}'

    def get(self, path: str) -> requests.Response:
        """Execute GET request to specified endpoint.

        Args:
            path: API endpoint path
        """
        return self.session.get(f'{self.base_url}{path}')

    def post(self, path: str, data: Dict[str, Any]) -> requests.Response:
        """Execute POST request with JSON payload.

        Args:
            path: API endpoint path
            data: Dictionary to send as JSON body
        """
        return self.session.post(f'{self.base_url}{path}', json=data)

    def put(self, path: str, data: Dict[str, Any]) -> requests.Response:
        """Execute PUT request with JSON payload.

        Args:
            path: API endpoint path
            data: Dictionary to send as JSON body
        """
        return self.session.put(f'{self.base_url}{path}', json=data)

    def delete(self, path: str) -> requests.Response:
        """Execute DELETE request.

        Args:
            path: API endpoint path
        """
        return self.session.delete(f'{self.base_url}{path}')


class TestEnd2End(unittest.TestCase):
    """End-to-end test suite for Student Management API.

    Test scenarios cover:
    - Student lifecycle (create/read/update/delete)
    - Course enrollment operations
    - Error handling
    """

    def setUp(self) -> None:
        """Initialize test environment by resetting database."""
        self.assertEqual(test_request.delete('/wipedatabase').status_code, 200)
        time.sleep(1)  # Allow index recreation

    def test_student(self) -> None:
        """Validate student CRUD lifecycle operations.

        Test steps:
        1. Create two student records
        2. Verify initial data integrity
        3. Update existing student
        4. Validate update persistence
        5. Test non-existent student handling
        6. Cleanup test data
        """
        # Create students
        self.assertEqual(test_request.put('/students/1', {'name': 'John Doe', 'courses': '90024'}).status_code, 201)
        self.assertEqual(test_request.put('/students/2', {'name': 'Jane Doe', 'courses': ['90024', '90059']}).status_code, 201)
        time.sleep(1)  # Allow ES indexing

        # Verify GET operations
        r = test_request.get('/students/1')
        self.assertEqual(r.status_code, 200)
        student_data = r.json()['_source']
        self.assertEqual(student_data['name'], 'John Doe')
        self.assertEqual(student_data['courses'], '90024')

        # Test array field handling
        r = test_request.get('/students/2')
        student_data = r.json()['_source']
        self.assertEqual(r.status_code, 200)
        self.assertEqual(student_data['name'], 'Jane Doe')
        self.assertEqual(student_data['courses'], ['90024', '90059'])

        # Update student
        update_response = test_request.put('/students/1', {'name': 'Bob Carr', 'courses': '90024'})
        self.assertEqual(update_response.status_code, 200)
        
        # Verify update
        r = test_request.get('/students/1')
        student_data = r.json()['_source']
        self.assertEqual(student_data['name'], 'Bob Carr')

        # Test non-existent resource
        self.assertEqual(test_request.get('/students/999').status_code, 404)

        # Cleanup
        self.assertEqual(test_request.delete('/students/1').status_code, 200)
        self.assertEqual(test_request.delete('/students/2').status_code, 200)

    def test_students(self) -> None:
        """Validate course-student relationship management.

        Test steps:
        1. Create prerequisite courses
        2. Create student with course enrollment
        3. Verify enrollment data integrity
        """
        # Create courses
        self.assertEqual(test_request.put('/courses/90024', {'name': 'Cloud Computing'}).status_code, 201)
        self.assertEqual(test_request.put('/courses/90059', {'name': 'Introduction to Programming'}).status_code, 201)
        
        # Create student
        self.assertEqual(test_request.put('/students/1', {'name': 'John Doe', 'courses': '90024'}).status_code, 201)
        
        # Verify enrollment
        r = test_request.get('/students')
        o = (r.json()['hits'])['hits']
        self.assertEqual(r.status_code, 200)
        self.assertEqual(o[0]['fields']['name'][0], 'John Doe')

    def test_course(self) -> None:
        """Validate course CRUD lifecycle operations.

        Test steps:
        1. Create two course records
        2. Verify initial data integrity
        3. Update existing course
        4. Validate update persistence
        5. Test non-existent course handling
        6. Cleanup test data
        """
        # Create courses
        self.assertEqual(test_request.put('/courses/90024', {'name': 'Cloud x Computing'}).status_code, 201)
        self.assertEqual(test_request.put('/courses/90059', {'name': 'Introduction to Programming'}).status_code, 201)
        time.sleep(1)

        # Verify GET operations
        r = test_request.get('/courses/90024')
        course_data = r.json()['_source']
        self.assertEqual(r.status_code, 200)
        self.assertEqual(course_data['name'], 'Cloud x Computing')

        # Test array field handling
        r = test_request.get('/courses/90059')
        course_data = r.json()['_source']
        self.assertEqual(r.status_code, 200)
        self.assertEqual(course_data['name'], 'Introduction to Programming')

        # Update course
        update_response = test_request.put('/courses/90024', {'name': 'Cloud Computing'})
        self.assertEqual(update_response.status_code, 200)
        
        # Verify update
        r = test_request.get('/courses/90024')
        course_data = r.json()['_source']
        self.assertEqual(course_data['name'], 'Cloud Computing')

        # Test non-existent resource
        self.assertEqual(test_request.get('/courses/999').status_code, 404)

        # Cleanup
        self.assertEqual(test_request.delete('/courses/90024').status_code, 200)
        self.assertEqual(test_request.delete('/courses/90059').status_code, 200)

    def test_courses(self) -> None:
        """Validate course listing operations.

        Test steps:
        1. Create two course records
        2. Verify course listing data integrity
        """
        # Create courses
        self.assertEqual(test_request.put('/courses/90024', {'name': 'Cloud Computing'}).status_code, 201)
        self.assertEqual(test_request.put('/courses/90059', {'name': 'Introduction to Programming'}).status_code, 201)
        time.sleep(1)

        # Verify course listing
        r = test_request.get('/courses')
        o = (r.json()['hits'])['hits']
        self.assertEqual(r.status_code, 200)
        self.assertEqual(o[0]['fields']['name'][0], 'Cloud Computing')
        self.assertEqual(o[1]['fields']['name'][0], 'Introduction to Programming')

    def test_courses_students(self) -> None:
        """Validate course-student relationship management.

        Test steps:
        1. Create prerequisite courses
        2. Create student with course enrollment
        3. Verify enrollment data integrity
        """
        # Create courses
        self.assertEqual(test_request.put('/courses/90024', {'name': 'Cloud Computing'}).status_code, 201)
        self.assertEqual(test_request.put('/courses/90059', {'name': 'Introduction to Programming'}).status_code, 201)
        self.assertEqual(test_request.put('/students/1', {'name': 'John Doe', 'courses': '90024'}).status_code, 201)
        self.assertEqual(test_request.put('/students/2', {'name': 'Jane Doe', 'courses': ['90024', '90059']}).status_code, 201)
        time.sleep(1)

        # Verify enrollment
        r = test_request.get('/courses/90024/students')
        o = (r.json()['hits'])['hits']
        self.assertEqual(r.status_code, 200)
        self.assertEqual(len(o), 2)
        self.assertEqual(o[0]['fields']['name'][0], 'Jane Doe')
        self.assertEqual(o[1]['fields']['name'][0], 'John Doe')

        # Verify enrollment for second course
        r = test_request.get('/courses/90059/students')
        o = (r.json()['hits'])['hits']
        self.assertEqual(len(o), 1)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(o[0]['fields']['name'][0], 'Jane Doe')

        # Verify non-existent course
        r = test_request.get('/courses/99999/students')
        o = (r.json()['hits'])['hits']
        self.assertEqual(r.status_code, 200)
        self.assertEqual(len(o), 0)

if __name__ == '__main__':
    test_request = HTTPSession('http', 'localhost', 9090)
    unittest.main()
